/* File: $Id: blockdev.cc,v 1.19 1998/02/25 16:17:56 morsiani Exp $ */

/****************************************************************************
 *
 * This module provides some utility classes for block devices handling.
 * They are: Block for block devices sectors/tape blocks representation; and
 * DriveParams for simulated disk devices performance parameters.
 *
 ****************************************************************************/

/****************************************************************************/
/* Inclusion of header files.                                               */
/****************************************************************************/

#include <stdio.h>

#include "h/const.h"
#include "h/types.h"
#include "h/blockdev.h"


/****************************************************************************/
/* Inclusion of imported declarations.                                      */
/****************************************************************************/

#include "e/utility.e"


/****************************************************************************/
/* Declarations strictly local to the module.                               */
/****************************************************************************/


/****************************************************************************/
/* Definitions to be exported.                                              */
/****************************************************************************/

// This class implements the block devices' 512 byte sectors/tape blocks.
// Each object contains a single buffer; methods are provided to read/write
// these blocks from/to real files and to access to the word-sized contents.
// This class is provided primarily to make DMA transfer easier and to 
// standardize block handling. 

class Block
{
	public:
		
		// This method returns an empty (unitialized) 512 byte Block
		Block(void);
		
		// Object deletion is done by default handler
		
		// This method fills a Block with file contents starting at "offset"
		// bytes from file start, as computed by caller.
		// Returns TRUE if read does not succeed, FALSE otherwise
		Boolean ReadBlock(FILE * blkFile, SWord offset);

		// This method writes Block contents in a file, starting at "offset"
		// bytes from file start, as computed by caller. Returns TRUE if
		// write does not succeed, FALSE otherwise
		Boolean WriteBlock(FILE * blkFile, SWord offset);
		
		// This method returns the Word contained in the Block at ofs (Word
		// items) offset, range [0..BLOCKSIZE - 1]. Warning: in-bounds
		// checking is leaved to caller
		Word getWord(unsigned int ofs);

		// This method fills with "value" the Word contained in the Block at
		// ofs (Word items) offset, range [0..BLOCKSIZE - 1]. Warning:
		// in-bounds checking is leaved to caller
		void setWord(unsigned int ofs, Word value);
			
	private:
		// Block contents
		Word blkBuf[BLOCKSIZE];
};


// This method returns an empty (unitialized) 512 byte Block
Block::Block(void)
{
}


// This method fills a Block with file contents starting at "offset"
// bytes from file start, as computed by caller.
// Returns TRUE if read does not succeed, FALSE otherwise
Boolean Block::ReadBlock(FILE * blkFile, SWord offset)
{
	if (fseek(blkFile, offset, SEEK_SET) == EOF)
		// already at EOF
		return(TRUE);
	else
		if (fread((void *)blkBuf, WORDLEN, BLOCKSIZE, blkFile) != BLOCKSIZE)
			// file too short
			return(TRUE);
		else
			// all OK
			return(FALSE);
}


// This method writes Block contents in a file, starting at "offset" bytes
// from file start, as computed by caller. Returns TRUE if write does not
// succeed, FALSE otherwise
Boolean Block::WriteBlock(FILE * blkFile, SWord offset)
{
	if (fseek(blkFile, offset, SEEK_SET) == EOF)
		// already at EOF
		return(TRUE);
	else
		if (fwrite((void *)blkBuf, WORDLEN, BLOCKSIZE, blkFile) != BLOCKSIZE)
			// some error occurred
			return(TRUE);
		else
			// all OK
			return(FALSE);
}


// This method returns the Word contained in the Block at ofs (Word items)
// offset, range [0..BLOCKSIZE - 1]. Warning: in-bounds checking is leaved
// to caller
Word Block::getWord(unsigned int ofs)
{
	if (ofs < BLOCKSIZE)
		return(blkBuf[ofs]);
	else
		return(MAXWORDVAL);
}


// This method fills with "value" the Word contained in the Block at ofs
// (Word items) offset, range [0..BLOCKSIZE - 1]. Warning: in-bounds
// checking is leaved to caller
void Block::setWord(unsigned int ofs, Word value)
{
	if (ofs < BLOCKSIZE)
		blkBuf[ofs] = value;
}


/****************************************************************************/


// This class contains the simulated disk drive geometry and performance
// parameters. They are filled by mkdev utility and used by DiskDevice class
// for detailed disk performance simulation.
// Position, min, max and default values, where applicable, are defined in 
// h/blockdev.h header file.
//
// Parameters are:
// number of cylinders;
// number of heads;
// number of sectors per track;
// disk rotation time in microseconds;
// average track-to-track seek time in microseconds;
// data % of sector (to compute inter-sector gap)

class DriveParams
{
	public:
		
		// This method reads from disk parameters from file header, builds a
		// DriveParams object, and returns the disk sectors start offset:
		// this allows to modify the parameters' size without changing the
		// caller.  If fileOfs returned is 0, something has gone wrong; file
		// is rewound after use
		DriveParams(FILE * diskFile, SWord * fileOfs);
		
		// Object deletion is done by default handler		

		// These methods return the corresponding geometry or performance 
		// figure 
		unsigned int getCylNum(void);
		unsigned int getHeadNum(void);
		unsigned int getSectNum(void);
		unsigned int getRotTime(void);
		unsigned int getSeekTime(void);
		unsigned int getDataSect(void);
		
		private:
			// parameter buffer
			unsigned int parms[DRIVEPNUM];
};


// This method reads from disk parameters from file header, builds a
// DriveParams object, and returns the disk sectors start offset: this
// allows to modify the parameters' size without changing the caller.
// If fileOfs returned is 0, something has gone wrong; file is rewound after
// use
DriveParams::DriveParams(FILE * diskFile, SWord * fileOfs) 
{
	SWord ret;
	unsigned int i;
	Block * blk = new Block();
	
	rewind(diskFile);
	if (blk->ReadBlock(diskFile, 0) || blk->getWord(0) != DISKFILEID)
		// errors in file reading or disk file magic number missing
		ret = 0;
	else
	{
		// if DISKFILEID is present all parameters should be correct; 
		// fills the object
		for (i = 0; i < DRIVEPNUM; i++)
			parms[i] = (unsigned int) blk->getWord(i + 1);
	
		rewind(diskFile);
		// sets the disk contents start position 
		ret = DRIVEPNUM + 1;
	}	
	delete blk;
	
	*fileOfs = ret;
}

// All the following methods return the corresponding geometry or
// performance figure

unsigned int DriveParams::getCylNum(void)
{
	return(parms[CYLNUM]);
}

unsigned int DriveParams::getHeadNum(void)
{
	return(parms[HEADNUM]);
}

unsigned int DriveParams::getSectNum(void)
{
	return(parms[SECTNUM]);
}

unsigned int DriveParams::getRotTime(void)
{
	return(parms[ROTTIME]);
}

unsigned int DriveParams::getSeekTime(void)
{
	return(parms[SEEKTIME]);
}

unsigned int DriveParams::getDataSect(void)
{
	return(parms[DATASECT]);
}


/****************************************************************************/
/* Definitions strictly local to the module.                                */
/****************************************************************************/

