/* File: $Id: device.e,v 1.19 1998/02/25 16:17:56 morsiani Exp $ */

/****************************************************************************
 *
 * External declarations from the corresponding module .cc
 *
 ****************************************************************************/

#define PRNTBUFSIZE 128
#define TERMBUFSIZE 128
#define DISKBUFSIZE 128
#define TAPEBUFSIZE 128
 
class SystemBus;
class SetupInfo;
class Block;
class DriveParams;


// Device class defines the interface to all device types, and represents
// the "uninstalled device" (NULLDEV) itself. Device objects are created and
// controlled by a SystemBus object, but also may be inspected by Watch if
// needed

class Device
{
	public:
	
		// This method creates a Device object with "coordinates" (interrupt
		// line, device number) clears device register, and links it to bus
		Device(SystemBus * busl, unsigned int intl, unsigned int dnum);
	
		virtual ~Device(void);

		// This method returns device type ID
		unsigned int Type(void);
	
		// This method is invoked by SystemBus when the required operation
		// scheduled on the device (using Event objects and queue) should be
		// completed: the default NULLDEV device has nothing to do, but
		// others do
		virtual unsigned int CompleteDevOp(void);

		// This method allows SystemBus to write into device register for
		// device: NULLDEV device register write has no effects, but other
		// devices will start performing required operations if COMMAND
		// register is written with proper codes
		virtual void WriteDevReg(unsigned int regnum, Word data);

		// This method returns the static buffer contained in each device
		// describing the current device status (operation performed, etc.).
		// NULLDEV devices are not operational
		virtual const char * getDevSStr(void);
		
		// This method returns a human-readable expression for completion time of
		// device operation under execution, if one
		virtual const char * getDevCTStr(void);				

		// This method allows to copy inputstr contents inside
		// TerminalDevice receiver buffer: not operational for all other
		// devices (NULLDEV included) and produces a panic message
		virtual void Input(const char * inputstr);

		// This method allows to load/unload tapes inside a TapeDevice. For
		// it, if tFName == NULL or EMPTYSTR, method returns TRUE if a new
		// tape may be loaded, FALSE otherwise; else, if tFName != NULL it
		// tries to load the specified tape file, returning completion
		// status. This method is not operational for other devices (NULLDEV
		// included) and produces a panic message
		virtual Boolean TapeLoad(const char * tFName);

		// This method returns the current value for device register field
		// indexed by regnum
		Word ReadDevReg(unsigned int regnum);

		// This method gets the current operational status for the device,
		// as set by user inside the simulation; a "not-working" device
		// fails all operations requested and reports proper error codes; a
		// NULLDEV always fail
		Boolean getDevNotWorking(void);

		// This method sets the operational status for the device inside the
		// simulation as user wishes; a "not-working" device fails all operations
		// requested and reports proper error codes; a NULLDEV always fail
		Boolean setDevNotWorking(Boolean cond);

		// This method returns old value of statChanged and sets it to
		// FALSE: this way any change into device status may be closely
		// monitored
		Boolean ClearDevStatChange(void);

		
	protected:

		// interrupt line and device number for device 
		unsigned int intL;
		unsigned int devNum;

		// device register structure
		Word reg[DEVREGLEN];

		// device type ID (see h/const.h)
		unsigned int dType;

		// controlling SystemBus object
		SystemBus * bus;

		// completion time TimeStamp for current operation (if one)
		TimeStamp * complTime;

		// device operational status 
		Boolean isWorking;

		// device status change flag
		Boolean statChanged;
};


/**************************************************************************/


// PrinterDevice class allows to emulate parallel character printer
// currently in use (see performance figures shown before). It uses the same
// interface as Device, redefining only a few methods' implementation: refer
// to it for individual methods descriptions. 
// It adds to Device data structure:
// a pointer to SetupInfo object containing printer log file name;
// a static buffer for device operation & status description;
// a FILE structure for log file access.

class PrinterDevice : public Device
{
	public:
		PrinterDevice(SystemBus * busl, SetupInfo * stp, unsigned int intl, unsigned int dnum);
		virtual ~PrinterDevice(void);
		virtual void WriteDevReg(unsigned int regnum, Word data);
		virtual unsigned int CompleteDevOp(void);
		virtual const char * getDevSStr(void);
		
	private:
 		// to be used for getting printer output file name...
		SetupInfo * setup;

		// log file handling
		FILE * prntFile;
		
		// static buffer
		char statStr[PRNTBUFSIZE];
};


/**************************************************************************/


// TerminalDevice class allows to emulate serial "dumb" terminal (see
// performance figures shown before). TerminalDevice may be split up into
// two independent sub-devices: "transmitter" and "receiver".
// It uses the same interface as Device, redefining only a few methods'
// implementation: refer to it for individual methods descriptions.
// It adds to Device data structure:
// a pointer to SetupInfo object containing terminal log file name;
// a static buffer for device operation & status description;
// a FILE structure for log file access;
// some structures for handling terminal transmitter and receiver.

class TerminalDevice : public Device
{
	public:
		TerminalDevice(SystemBus * busl, SetupInfo * stp, unsigned int intl, unsigned int dnum);
		virtual ~TerminalDevice(void);
		virtual void WriteDevReg(unsigned int regnum, Word data);
		virtual unsigned int CompleteDevOp(void);
		virtual const char * getDevSStr(void);
		virtual const char * getDevCTStr(void);
		virtual void Input(const char * inputstr);
		
	private:
		// for log file name
		SetupInfo * setup;
		// for log file handling
		FILE * termFile;
		
		// receiver buffer and pointer to first character to receive from it
		char * recvBuf;
		unsigned int recvBp;
		
		// static buffer for receiver
		char recvStatStr[TERMBUFSIZE];
		
		// static buffer for transmitter
		char tranStatStr[TERMBUFSIZE];
		
		// completion time TimeStamp for current receiver operation (if one)
		TimeStamp * recvCTime;

		// completion time TimeStamp for current transmitter operation (if one)
		TimeStamp * tranCTime;
		
		// receiver operation pending flag
		Boolean recvIntPend;		
		
		// transmitter operation pending flag
		Boolean tranIntPend;
};


/**************************************************************************/


// DiskDevice class allows to emulate a disk drive: each 512 byte sector it
// contains is identified by (cyl, head, sect) set of disk coordinates;
// (geometry and performance figures are loaded from disk image file). 
// Operations on sectors (R/W) require previous seek on the desired cylinder.
// It also contains a sector buffer of one sector to speed up operations.
//
// It uses the same interface as Device, redefining only a few methods'
// implementation: refer to it for individual methods descriptions.
//
// It adds to Device data structure:
// a pointer to SetupInfo object containing printer log file name;
// a static buffer for device operation & status description;
// a FILE structure for disk image file access;
// a set of disk parameters (read from disk image file header);
// a Block object for file handling;
// some items for performance computation.

class DiskDevice : public Device
{
	public:
		DiskDevice(SystemBus * busl, SetupInfo * stp, unsigned int intl, unsigned int dnum);
		virtual ~DiskDevice(void);
		virtual void WriteDevReg(unsigned int regnum, Word data);
		virtual unsigned int CompleteDevOp(void);
		virtual const char * getDevSStr(void);
		
	private:
	
		// to get disk image file name
		SetupInfo * setup;
		
		// to handle it
		FILE * diskFile;
		
		// static buffer
		char statStr[DISKBUFSIZE];
		
		// sector buffer and coordinates on disk (cyl, head, sect)
		Block * diskBuf;
		unsigned int cylBuf, headBuf, sectBuf;
		
		// start of disk image inside file (after header)
		SWord diskOfs;
		
		// disk performance parameters
		DriveParams * diskP;
		
		// sector underhead time in ticks
		Word sectTicks;

		// current cylinder
		unsigned int currCyl;
};


/**************************************************************************/


// TapeDevice class allows to emulate removable cartridge tape drives. 
// Individual tapes may be loaded and unloaded, rewound and read 
// (see performance figures shown before). TapeDevice uses the same
// interface as Device, redefining only a few methods' implementation: refer
// to it for individual methods descriptions.
// It adds to Device data structure:
// a pointer to SetupInfo object containing tape cartridge log file name;
// a static buffer for device operation & status description;
// a FILE structure for log file access;
// a Block object for file handling.

class TapeDevice : public Device
{
	public:
		TapeDevice(SystemBus * busl, SetupInfo * stp, unsigned int intl, unsigned int dnum);
		virtual ~TapeDevice(void);
		virtual void WriteDevReg(unsigned int regnum, Word data);
		virtual unsigned int CompleteDevOp(void);
		virtual const char * getDevSStr(void);
		virtual Boolean TapeLoad(const char * tFName);

	private:
		// to get tape cartridge file name
		SetupInfo * setup;
		
		// to access tape image file
		FILE * tapeFile;
		char * tapeFName;
		
		// to read tape blocks and know current position (starts with block 0)
		Block * tapeBlk;
		unsigned int tapeBp;
		
		// loaded cartridge flag
		Boolean tapeLoaded;
		
		// static buffer
		char statStr[TAPEBUFSIZE];
};
