/* File: $Id: setup.cc,v 1.21 2004/03/14 10:41:55 morsiani Exp morsiani $ */

/****************************************************************************
 *
 * This module contains the SetupInfo class definition. Inside a SetupInfo
 * object, all configuration paramenters relevant to simulation have their
 * place.
 *
 ****************************************************************************/

/****************************************************************************/
/* Inclusion of header files.                                               */
/****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>	// kMcC ;)

#include "h/const.h"
#include "h/types.h"
#include "h/blockdev.h"
#include "h/aout.h"


/****************************************************************************/
/* Inclusion of imported declarations.                                      */
/****************************************************************************/

#include "e/utility.e"


/****************************************************************************/
/* Declarations strictly local to the module.                               */
/****************************************************************************/

// default startup device configuration
HIDDEN unsigned int defaultDev[DEVINTUSED][DEVPERINT] =	{	
															{
																DISKDEV,
																NULLDEV, 
																NULLDEV,
																NULLDEV
															},
															{
																TAPEDEV,
																NULLDEV, 
																NULLDEV,
																NULLDEV
															},
															{	
																NULLDEV,												 
																NULLDEV, 
																NULLDEV,
																NULLDEV
															},
															{
																PRNTDEV,
																NULLDEV, 
																NULLDEV,
																NULLDEV
															},
															{
																TERMDEV,
																TERMDEV,												
																TERMDEV,
																TERMDEV
															}
														};


// default device class file names
HIDDEN char * defltFName[DEVINTUSED] =	{	"disk",
											"tape",
											"nodev",
											"printer",
											"term"
										};


// default .rom file names (Bootstrap ROM excluded)
HIDDEN char * defltROMFName[ROMSNUM] =	{	"",
											"kernel",
											"bios",
											"kernel"
										};

										
// default Bootstrap ROM file names (one for boot type)
HIDDEN char * bootROMFName[BOOTTYPES] =	{	"coreboot",
											"tapeboot", 
											"diskboot"
										};


// default .rom file extension										
HIDDEN char * romFileExt[ROMSNUM] =	{	BIOSFILETYPE, 
										COREFILETYPE,
										BIOSFILETYPE,
										STABFILETYPE
									};	

// default device class for interrupt lines available for devices										
HIDDEN unsigned int devClass[DEVINTUSED] = 	{		DISKDEV,
													TAPEDEV,
													ETH0DEV,
													PRNTDEV,
													TERMDEV
											};

// default device class name
HIDDEN char * devTypeStr[DEVINTUSED + 1] = 	{	"NULLDEV ",
												"DISK    ",
												"TAPE    ",
												"NETWORK ",
												"PRINTER ",
												"TERMINAL"
											};

// device status
HIDDEN char * active[2] =	{	"INACTIVE",
								"ACTIVE  "
							};

// static buffer 
#define STRBUFSIZE	256
HIDDEN char strbuf[STRBUFSIZE];


/****************************************************************************/
/* Definitions to be exported.                                              */
/****************************************************************************/

// This class implements SetupInfo objects. Each contains all configuration
// information for every other object in the program which may need it: see
// object contents.

class SetupInfo
{
	public:
		// This method builds a new SetupInfo object and set ROM file
		// configuration depending on boot type. If boot type is "core
		// boot", a kernel image must be loaded in RAM memory; for any other
		// kind of boot type, no kernel image needs to be loaded, but a
		// different Bootstrap ROM does.  Other system configuration
		// parameters are set to default, and devices are activated or
		// deactivated accordingly with file availability and default
		// configuration
		SetupInfo(unsigned int bType, const char * bootFile);

		// This method deletes all SetupInfo associated structures
		~SetupInfo(void);


		// the following methods allow to get the current configuration
		// values; their meaning and arguments are self-explanatory
		
		unsigned int getSpeed(void);
		Word getRamSize(void);
		const char * getROMFileName(unsigned int index);
		const char * DevIsActiveStr(unsigned int intl, unsigned int dnum);
		unsigned int getBootType(void);
		unsigned int getDevClass(unsigned int intl);
		const char * getDevClassStr(unsigned int intl);
		unsigned int getDevType(unsigned int intl, unsigned int dnum);
		const char * getDevTypeStr(unsigned int intl, unsigned int dnum);
		const char * getDevFileName(unsigned int intl, unsigned int dnum);
		Word getSymbolTableASID(void);
		
		
		// all these methods allow to update the current configuration values;
		// their meaning and arguments are almost self-explanatory
		
		void setSpeed(unsigned int val);
		void setRamSize(Word val);
		Boolean setROMFileName(unsigned int index, const char * fName);
		
		// This method detects if a device file for the specificated device
		// exists and is correct; if necessary it "activates" the device. 
		// It returns TRUE and set the file name to the new one if all is
		// OK, and returns FALSE (with no change in device file name or
		// status) otherwise.  If fname is NULL or empty, it simply sets the
		// device to "deactivated" and returns always TRUE.
		Boolean setDevFileName(unsigned int intl, unsigned int dnum, const char * fName);
		
		void setBootType(unsigned int bType);
		void setSymbolTableASID(Word asid);
		
	private:
	
		// processor speed in MHz
		unsigned int speed;
		
		// RAM size (in 4KB frames) 
		Word ramSize;

		// ASID associated to current symbol table		
		Word symTabASID;

		// boot type ("core", from disk, from tape)		
		unsigned int bootType;
		
		// device type current configuration 
		unsigned int devType[DEVINTUSED][DEVPERINT];
		
		// device file names
		char * devFName[DEVINTUSED][DEVPERINT];
		
		// ROM & symbol table file names 
		char * romFName[ROMSNUM];
		
		// This method tests device file validity. A device file is valid
		// when: if it is a tape or disk file, it contains the correct magic
		// number; if it is another kind of device, if file is writable
		// (since it is just a log file).  It returns TRUE or FALSE
		// according to file validity
		Boolean validFileMagic(Word tag, const char * fName);
};

// This method builds a new SetupInfo object and sets ROM file configuration
// depending on boot type. If boot type is "core boot", a kernel image must
// be loaded in RAM memory; for any other kind of boot type, no kernel image
// needs to be loaded, but a different Bootstrap ROM does.
// Other system configuration parameters are set to default, and devices are
// activated or deactivated accordingly with file availability and default
// configuration
// ROMDIR (set at "make" time) allows to specify a default location for .rom
// files
SetupInfo::SetupInfo(unsigned int bType, const char * bootFile)
{
	unsigned int i, j;
	char * strp, * mpsdirp;
	char fileName[STRBUFSIZE];
	char fileDir[STRBUFSIZE];
	
	speed = MINSPEED;
	ramSize = STARTRAM;
	
	symTabASID = KERNID;
	
	getcwd(fileDir, STRBUFSIZE);

	// test device file availability and sets configuration	
	for (i = 0; i < DEVINTUSED; i++)
		for (j = 0; j < DEVPERINT; j++)
		{
			// clears device conditions
			devType[i][j] = NULLDEV;
			
			devFName[i][j] = new char[strlen(EMPTYSTR) + 1];
			strcpy(devFName[i][j], EMPTYSTR);
			// devFName should never be NULL
			
			if (defaultDev[i][j] != NULLDEV)
			{
				sprintf(fileName, "%s/%s%u%s",fileDir, defltFName[i], j, MPSFILETYPE);
				// tries to activate device: setDevFileName will emit a warning 
				// message if device could not be activated
				setDevFileName(i, j, fileName);
			}
		}	

	// sets boot type
	if (bType < BOOTTYPES)
		bootType = bType;
	else
		Panic("Invalid boot type in SetupInfo::SetupInfo()");

		
	// clears pointers
	for (i = 0; i < ROMSNUM; i++)
		romFName[i] = NULL;

	// tries to locate boot ROM file and load it: if fails, sets to
	// consistent state
	if ((mpsdirp = getenv("MPSDIR")) != NULL)
		sprintf(fileName, "%s/%s%s", mpsdirp, bootROMFName[bootType], romFileExt[BOOTINDEX]);
	else
	{
#ifdef ROMDIR
		sprintf(fileName, "%s/%s%s", ROMDIR, bootROMFName[bootType], romFileExt[BOOTINDEX]);
#else
		sprintf(fileName, "%s/%s%s", fileDir, bootROMFName[bootType], romFileExt[BOOTINDEX]);
#endif
	}

	if (!setROMFileName(BOOTINDEX, fileName))
		setROMFileName(BOOTINDEX, NULL);

	if (bootFile == NULL)
		// sets up with defaults
		sprintf(fileName, "%s/%s%s", fileDir, defltROMFName[COREINDEX], romFileExt[COREINDEX]);
	else
		sprintf(fileName, "%s",bootFile);

	// tries to locate & load core image: if fails, sets to consistent state	
	// this is done testing for file validity: if validation fails, will set
	// file name to EMPTYSTR, so other modules will know this
	if (bootType != COREBOOT || !setROMFileName(COREINDEX, fileName))		
		setROMFileName(COREINDEX, NULL);			

	// sets up and tries to load bios file: if fails, sets to consistent state
	if ((mpsdirp = getenv("MPSDIR")) != NULL)	
		sprintf(fileName, "%s/%s%s", mpsdirp, defltROMFName[BIOSINDEX], romFileExt[BIOSINDEX]);	
	else
	{
#ifdef ROMDIR
		sprintf(fileName, "%s/%s%s", ROMDIR, defltROMFName[BIOSINDEX], romFileExt[BIOSINDEX]);
#else
		sprintf(fileName, "%s/%s%s", fileDir, defltROMFName[BIOSINDEX], romFileExt[BIOSINDEX]);	
#endif
	}
	
	if (!setROMFileName(BIOSINDEX, fileName))
		setROMFileName(BIOSINDEX, NULL);

	//identifies and try to load symbol table file 
	if (bootFile == NULL) 
		// set to defaults
		sprintf(fileName, "%s/%s%s", fileDir, defltROMFName[STABINDEX], romFileExt[STABINDEX]);
	else
	{
		// identify kernel core extension
		strcpy(strbuf, bootFile);
		if ((strp = strstr(strbuf, romFileExt[COREINDEX])) != NULL)
		{
			*strp = EOS;
			sprintf(fileName, "%s%s", strbuf, romFileExt[STABINDEX]);
		}
		else
			// back to defaults	
			sprintf(fileName, "%s/%s%s", fileDir, defltROMFName[STABINDEX], romFileExt[STABINDEX]);
	}
	// tries to set it in consistent way
	if (!setROMFileName(STABINDEX, fileName))
			setROMFileName(STABINDEX, NULL);
}


// This method deletes all SetupInfo associated structures
SetupInfo::~SetupInfo(void)
{
	unsigned int i, j;
	
	for (i = 0; i < DEVINTUSED; i++)
		for (j = 0; j < DEVPERINT; j++)
			if (devFName[i][j] != NULL)
				delete (devFName[i][j]);

	for (i = 0; i < ROMSNUM; i++)
		if (romFName[i] != NULL)
			delete (romFName[i]);
}

//
// the following methods allow to get the current configuration values, or
// to update them; their meaning and arguments are self-explanatory
//                

unsigned int SetupInfo::getSpeed(void)
{
	return(speed);
}

void SetupInfo::setSpeed(unsigned int val)
{
	speed = val;
}

Word SetupInfo::getRamSize(void)
{
	return(ramSize);
}

void SetupInfo::setRamSize(Word val)
{
	ramSize = val;
}

const char * SetupInfo::getROMFileName(unsigned int index)
{
	if (index < ROMSNUM)
		return(romFName[index]);
	else
	{
		Panic("Unknown ROM file index in SetupInfo::getROMFileName()");
		return(NULL);
	}
}

Boolean SetupInfo::setROMFileName(unsigned int index, const char * fName)
{
	Word tag = 0;
	
	// switch tests index validity too
	switch (index)
	{
		case COREINDEX:
			tag = COREFILEID;
			break;
		
		case BIOSINDEX:
		case BOOTINDEX:
			tag = BIOSFILEID;
			break;
		
		case STABINDEX:
			tag = STABFILEID;
			break;
		
		default:
			Panic("Unknown ROM file index in SetupInfo::setROMFileName()");
			break;
	}
	if (fName == NULL || SAMESTRING(fName, EMPTYSTR))
	{
		// setting to empty
		if (romFName[index] != NULL)
			delete (romFName[index]);
		romFName[index] = new char[strlen(EMPTYSTR) + 1];
		strcpy(romFName[index], EMPTYSTR);
		return(TRUE);
	}
	else
		if (validFileMagic(tag, fName))
		{
			if (romFName[index] != NULL)
				delete (romFName[index]);
			romFName[index] = new char[strlen(fName) + 1];
			strcpy(romFName[index], fName);
			return(TRUE);
		}
		else
		{
			// invalid file: no change
				ShowAlert("ROM file not found or invalid", "file name:",fName);
			return(FALSE);
		}
}

unsigned int SetupInfo::getDevClass(unsigned int intl)
{
	if (intl < DEVINTUSED)
		return(devClass[intl]);
	else
	{
		Panic("Invalid interrupt line in SetupInfo::getDevClass()");
		return(NULLDEV);
	}
}

const char * SetupInfo::getDevClassStr(unsigned int intl)
{
	if (intl < DEVINTUSED)
		return(devTypeStr[devClass[intl]]);
	else
	{
		Panic("Invalid interrupt line in SetupInfo::getDevClassStr()");
		return(NULLDEV);
	}
}

unsigned int SetupInfo::getDevType(unsigned int intl, unsigned int dnum)
{
	if (intl < DEVINTUSED && dnum < DEVPERINT)
		return(devType[intl][dnum]);
	else
	{
		Panic("Invalid interrupt/device line in SetupInfo::getDevType()");
		return(NULLDEV);
	}
}

const char * SetupInfo::getDevTypeStr(unsigned int intl, unsigned int dnum)
{
	if (intl < DEVINTUSED && dnum < DEVPERINT)
		return(devTypeStr[devType[intl][dnum]]);
	else
	{
		Panic("Invalid interrupt/device line in SetupInfo::getDevTypeStr()");
		return(NULL);
	}
}

const char * SetupInfo::getDevFileName(unsigned int intl, unsigned int dnum)
{
	if (intl < DEVINTUSED && dnum < DEVPERINT)
		return(devFName[intl][dnum]);
	else
	{
		Panic("Invalid interrupt/device line in SetupInfo::getDevFileName()");
		return(NULL);
	}
}


// This method detects if a device file for the specificated device exists
// and is correct; if necessary it "activates" the device. 
// It returns TRUE and set the file name to the new one if all is OK, and
// returns FALSE (with no change in device file name or status) otherwise.
// If fname is NULL or empty, it simply sets the device to "deactivated" and
// returns always TRUE
Boolean SetupInfo::setDevFileName(unsigned int intl, unsigned int dnum, const char * fName)
{
	Word tag = MAXWORDVAL;
	
	if (dnum >= DEVPERINT)
	{
		Panic("Invalid device line in SetupInfo::setDevFileName()");
		return(FALSE);
	}
	// else dnum is ok
	// switch tests intl, & checks dev type too
	switch(getDevClass(intl))
	{
		case PRNTDEV:
		case TERMDEV:
		case ETH0DEV:
			tag = 0UL;
			break;
	
		case DISKDEV:
			tag = DISKFILEID;
			break;
			
		case TAPEDEV:
			tag = TAPEFILEID;
			break;
	
		default:
			Panic("Unknown device class in SetupInfo::setDevFileName()");
			break;
	}
	
	if (fName == NULL || SAMESTRING(fName, EMPTYSTR))
	{
		// deactivate device
		if (devFName[intl][dnum] != NULL)
			delete (devFName[intl][dnum]);
		devFName[intl][dnum] = new char[strlen(EMPTYSTR) + 1];
		strcpy(devFName[intl][dnum], EMPTYSTR);
		devType[intl][dnum] = NULLDEV;
		return(TRUE);
	}
	else
		if (validFileMagic(tag, fName))
		{	
			// copies file name & activates device
			if (devFName[intl][dnum] != NULL)
				delete (devFName[intl][dnum]);
			devFName[intl][dnum] = new char[strlen(fName) + 1];
			strcpy(devFName[intl][dnum], fName);
			devType[intl][dnum] = getDevClass(intl);
			return(TRUE);
		}
		else
		{	
			// no change
			// emits a warning message
			sprintf(strbuf, "File for Device %s  Int: %u  Dev: %u  not found or invalid",  getDevClassStr(intl), \
			intl + DEVINTBASE,  dnum);
			ShowAlert(strbuf, "file name:", fName);
			return(FALSE);
		}
}

const char * SetupInfo::DevIsActiveStr(unsigned int intl, unsigned int dnum)
{
	if (devType[intl][dnum] == NULLDEV)
		return(active[FALSE]);
	else
		return(active[TRUE]); 
	
} 

unsigned int SetupInfo::getBootType(void)
{
	return(bootType);
}

void SetupInfo::setBootType(unsigned int bType)
{
	char fileName[STRBUFSIZE], * mpsdirp;
	
	if (bootType < BOOTTYPES)
	{
		bootType = bType;

		// tries to locate boot ROM file that matches bootType and load it: if fails, sets to consistent state 
	if ((mpsdirp = getenv("MPSDIR")) != NULL)
		sprintf(fileName, "%s/%s%s", mpsdirp, bootROMFName[bootType], romFileExt[BOOTINDEX]);				
	else
	{
#ifdef ROMDIR
			sprintf(fileName, "%s/%s%s", ROMDIR, bootROMFName[bootType], romFileExt[BOOTINDEX]);		
#else
			getcwd(strbuf, STRBUFSIZE);
			sprintf(fileName, "%s/%s%s", strbuf, bootROMFName[bootType], romFileExt[BOOTINDEX]);
#endif
	}
		if (!setROMFileName(BOOTINDEX, fileName))
			setROMFileName(BOOTINDEX, NULL);
	}
	else
		Panic("Invalid boot type in SetupInfo::setBootType()");

}

Word SetupInfo::getSymbolTableASID(void)
{
	return(symTabASID);
}

void SetupInfo::setSymbolTableASID(Word asid)
{
	symTabASID = asid;
}


// This method tests device file validity. A device file is valid when: if
// it is a tape or disk file, it contains the correct magic number; if it is
// another kind of device, if file is writable (since it is simply a log
// file)
// It returns TRUE or FALSE accordingly to file validity

Boolean SetupInfo::validFileMagic(Word tag, const char * fName)
{
	Word fTag;
	FILE * testF = NULL;
	Boolean ret = TRUE;
	
	if (tag == 0UL)
		// just testing file write access, without changing the file itself
	{
		if ((testF = fopen(fName, "a")) == NULL)
			// file is not accessible
			ret = FALSE;
		// else file is ok
	}	
	else
		// tag != 0UL
		if ((testF = fopen(fName, "r")) == NULL || fread((void *) &fTag, WORDLEN, 1, testF) != 1 || fTag != tag)
			// file empty/invalid
			ret = FALSE;
		// else file matches tag
		
	if (testF != NULL)
		fclose(testF);
			
	return(ret);
}
 
